



//*******************************************************************************//
//* 				Pliki doczane												*//
//*******************************************************************************//
#include <stdlib.h>									// Standardowa C
#include <avr/io.h>									// IO
#include <avr/sfr_defs.h>							// SFR, _BV(bit)
#include <avr/pgmspace.h>							// Flash
#include <avr/interrupt.h>							// wektory IRQ
#include <avr/wdt.h> 								// WDG
#include <avr/eeprom.h>								// Eeprom
#include <compat/deprecated.h>						// sbi, cbi, inb, outb, outp
//#include <math.h> 								// Zmiennoprzecinkowe
//#include <util/setbaud.h>							// Uart
//#include <util/crc16.h>							// Crc16, Xmodem, Dallas
#define ebi(port,bit) ( port ^= (1 << (bit)))		// Negowanie bitu na porcie
#define CODE PROGMEM								// Stale w Flash dla parametru CODE i PROGMEM
#define NOINIT __attribute__ ((section(".noinit")))




//*******************************************************************************//
//* 				Definicje													*//
//*******************************************************************************//
#define LedGreen	PB2
#define LedRed		PB1
#define LedYellow	PB0
#define Sin			( PINB & (1 << (PB4) ) )
#define TorBusy		( PINB & (1 << (PB3) ) )

#define GreenOff	PORTB |=  (1 << LedGreen )		// Wycz zielone
#define GreenOn		PORTB &= ~(1 << LedGreen )		// Wcz zielone
#define GreenNeg	PORTB ^=  (1 << LedGreen )		// Neguje stan zielonego
#define GreenBlink	Blink |=  (1 << LedGreen )		// Wcza miganie zielonego
#define GreenIn		( PORTB & (1 << LedGreen ) )

#define RedOff		PORTB |=  (1 << LedRed )		// Wycz czerwone
#define RedOn		PORTB &= ~(1 << LedRed )		// Wcz czerwone
#define RedIn		( PORTB & (1 << LedRed ) )

#define YellowOff	PORTB |=  (1 << LedYellow )		// Wycz te
#define YellowOn	PORTB &= ~(1 << LedYellow )		// Wcz te
#define YellowNeg	PORTB ^=  (1 << LedYellow )		// Neguje stan tego
#define YellowBlink	Blink |=  (1 << LedYellow )		// Wcza miganie tego
#define YellowIn	( PORTB & (1 << LedYellow ) )



//-------------------------- Zmienne, Funkcje  ------------------------------//
volatile char Sygnal=0,Blink=0;
void SetSygnal(char Syg); 


//**********************************************************************//
//						Program gwny			 						//
//**********************************************************************//
int main()
{

	DDRB = 0x07;								// PB 0, 1, 2 wyjciami
	PORTB = 0xFF;								// Wyjscia w stan wysoki, wejcia z podciganiem



	TCCR0B = 0x09;								// Preskaler na 1024 (7,812Hz)
	TIMSK = (1 << TOIE0);						// Zezwolenie na IRQ od 
	sei();										// Wcz przerwania timera






	while(1)
		{
		if ( TorBusy == 0) { SetSygnal(1); }	// Jeli tor zajety, to stgna stop
		else
			{
			if ( Sin != 0 ){ SetSygnal(2); }	// Jeli na poprzednim nie ma sygnau S1 (Stop) to daj S2
			if ( Sin == 0 ){ SetSygnal(5); }	// Jeli na poprzednim S1 to daj S5
			}
		}



	return(0);
}




//----------------------------------------------------------------------//
//					Wykrywanie zmiany sygnaw							//
//
// S1	czerwone				(stj)
// S2	zielone					(jazda z najwiksz dozwolon prdkoci)
// S5	pomaraczowe				(nastpny semafor pokazuje sygna stj)
// S3	migajce zielone				(max 160km, nastpny 100km, dwa odstpy wolne)
// S4	migajce pomaraczowe			(jazda zwolniona do 40 lub 60km)
// S8	migajce pomaraczowe + pomaraczowe + zielony pas	(100km nastpny 40 lub 60km)
// S13	dwa pomaraczowe			(40km, nastpny semafor stj)
// S10	zielone + pomaraczowe			(40km, nastpny max)
// S11	migajce zielone + pomaraczowe		(40km, nastpny 100km)
// S12	migajce pomaraczowe + pomaraczowe	(40km, nastpny 40 lub 60km)
//----------------------------------------------------------------------//
void SetSygnal(char Syg)
{


	Blink = 0;
	switch( Syg)
		{
		case(1):								// S1 - Stj
			GreenOff; 
			YellowOff;
			RedOn;
		break;
		case(2):								// S2 - jazda z najwiksz dozwolon prdkoci
			RedOff;
			YellowOff;
			GreenOn;
		break;
		case(3):								// S3 - max 160km, nastpny 100km, dwa odstpy wolne
			RedOff;
			YellowOff;
			GreenBlink;
		break;
		case(4):								// S4 - jazda zwolniona do 40 lub 60km
			RedOff;
			GreenOff;
			YellowBlink;
		break;
		case(5):								// S5 - nastpny semafor pokazuje sygna stj
			RedOff;
			GreenOff;
			YellowOn;
		break;
		}


}



//----------------------------------------------------------------------//
//					Obsuga IRQ od timera 0								//
//----------------------------------------------------------------------//
ISR( TIM0_OVF_vect )
{
static unsigned int Dziel1s=2;

	TCNT0 = 0;									// aduj timer 


	//-------------- Dzielnik do migania sygnaami ---
	if (--Dziel1s == 0)
		{
		Dziel1s = F_CPU / 1024;					// Dzielnik dla 1 sekundy

		if ( (Blink & (1 << LedYellow) ) != 0 ){ YellowNeg; }
		if ( (Blink & (1 << LedGreen) ) != 0 ) { GreenNeg; }
		}


}





